searchData={"items":[{"type":"module","title":"IIIFImagePlug.V3","doc":"This plug implements the IIIF Image API version 3 (see also https://iiif.io/api/image/3.0/).","ref":"IIIFImagePlug.V3.html"},{"type":"module","title":"Options - IIIFImagePlug.V3","doc":"### `:identifier_to_path_callback` required\nAn arity 1 callback function that resolves a given IIIF identifier to a file path (string).\n\n### `:max_width` (default: `10000`)\nThe maximum image width the plug will serve.\n\n### `:max_height` (default: `10000`)\nThe maximum image height the plug will serve.\n\n### `:max_area` (default: `100000000`)\nThe maximum amount of image pixels the plug will serve (does not necessarily have to be `max_width` * `max_height`).\n\n### `:preferred_formats` (default: `[:jpg]`)\nThe [preferred formats](https://iiif.io/api/image/3.0/#55-preferred-formats) to be used for your service.\n\n### `:extra_formats` (default: `[:png, :webp, :tif]`)\nThe [extra formats](https://iiif.io/api/image/3.0/#57-extra-functionality) your service can deliver. Note that TIF files\nhave to be buffered before they are sent, so large images might cause issues.\n\n### `:identifier_to_rights_callback` (optional)\nAn arity 1 callback function that returns a [rights](https://iiif.io/api/image/3.0/#56-rights) statement for a given identifier.\n\n### `:identifier_to_part_of_callback` (optional)\nAn arity 1 callback function that returns a list of [part of](https://iiif.io/api/image/3.0/#58-linking-properties) properties for a given identifier.\n\n### `:identifier_to_see_also_callback` (optional)\nAn arity 1 callback function that returns a list of [see also](https://iiif.io/api/image/3.0/#58-linking-properties) properties for a given identifier.\n\n### `:identifier_to_service_callback` (optional)\nAn arity 1 callback function that returns a list of [service](https://iiif.io/api/image/3.0/#58-linking-properties) properties for a given identifier.\n\n### `:status_callbacks` (optional)\nA map where each key is a HTTP status code (integer), and each value an arity 2 callback that can be used to replace the plug's default response. Each\ncallback should accept a plug as its first parameter and a Map (containing the error message) as its second parameter.\n\n### `:scheme` (optional)\nOverride the scheme evaluated from the `%Plug.Conn{}`, useful if your Elixir app runs behind a proxy.\n\n### `:host` (optional)\nOverride the host evaluated from the `%Plug.Conn{}`, useful if your Elixir app runs behind a proxy.\n\n### `:port` (optional)\nOverride the port evaluated from the `%Plug.Conn{}`, useful if your Elixir app runs behind a proxy.","ref":"IIIFImagePlug.V3.html#module-options"},{"type":"function","title":"IIIFImagePlug.V3.call/2","doc":"","ref":"IIIFImagePlug.V3.html#call/2"},{"type":"function","title":"IIIFImagePlug.V3.construct_id_url/2","doc":"","ref":"IIIFImagePlug.V3.html#construct_id_url/2"},{"type":"function","title":"IIIFImagePlug.V3.init/1","doc":"","ref":"IIIFImagePlug.V3.html#init/1"},{"type":"module","title":"IIIFImagePlug.V3.Data","doc":"Produces image data based on the given IIIF parameters and Plug settings.","ref":"IIIFImagePlug.V3.Data.html"},{"type":"function","title":"IIIFImagePlug.V3.Data.get/6","doc":"Get a transformed version of the image defined by `identifier` and the other provided parameters,\nvalidated against the plug's settings.\n\nReturns\n- `{%Vix.Vips.Image{}, format}` on success, where format is one of the preferred or extra format atoms defined\nin the plug settings.\n- `{:error, reason}` if a parameter was invalid.","ref":"IIIFImagePlug.V3.Data.html#get/6"},{"type":"module","title":"IIIFImagePlug.V3.Information","doc":"Produces image metadata (info.json).","ref":"IIIFImagePlug.V3.Information.html"},{"type":"function","title":"IIIFImagePlug.V3.Information.get/3","doc":"Get the IIIF `info.json` for the image defined by `identifier` with respect to the plug's settings.\n\nReturns\n- `info` as an Elixir map on success.\n- `{:error, :no_file}` if the identifier did not point to a file.\n- `{:error, :no_image_file}` if the identifier did not point to a file.","ref":"IIIFImagePlug.V3.Information.html#get/3"},{"type":"module","title":"IIIFImagePlug.V3.Settings","doc":"","ref":"IIIFImagePlug.V3.Settings.html"},{"type":"extras","title":"Overview","doc":"[![CI status](https://github.com/dainst/iiif_image_plug/actions/workflows/ci.yml/badge.svg)](https://github.com/dainst/iiif_image_plug/actions/workflows/ci.yml)\n\n- [hex](https://hex.pm/packages/iiif_image_plug)\n- [docs](https://hexdocs.pm/iiif_image_plug/)\n\n# IIIF Image Plug\n\nAn Elixir [plug](https://hexdocs.pm/plug/readme.html) implementing the _International Image Interoperability Framework_ ([IIIF](https://iiif.io/)) image API specification. \n\n- The goal of IIIF is to define a standardised REST API for serving high resolution images (art, photographes or archival material published by museums, universities and similar institutions).\n- There exist [several](https://iiif.io/get-started/iiif-viewers/) generic Javascript IIIF viewers that utilize this API to allow for optimized viewing (dynamic loading of parts of the image data based on zoom level/viewport).\n- WebGIS Javascript libraries like [leaflet](https://github.com/mejackreed/Leaflet-IIIF) or [OpenLayers](https://openlayers.org/en/latest/examples/iiif.html) support IIIF in one way or the other.\n- For the time beeing only (the current) Image API 3.0 is implemented, check out the IIIF [documentation](https://iiif.io/api/image/3.0/) for its capabilities.\n- The image processing is handled by [libvips](https://www.libvips.org/) via [Vix](https://hex.pm/packages/vix).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed\nby adding `iiif_image_plug` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:iiif_image_plug, \"~> 0.4.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Overview","doc":"Assuming you want to serve IIIF in your plug based server at \"/iiif/v3\", add a forward route like this: \n\n```elixir\n  forward(\"/iiif/v3\",\n    to: IIIFImagePlug.V3,\n    init_opts: %{\n      identifier_to_path_callback: &ImageStore.identifier_to_path/1\n    }\n  )\n```","ref":"readme.html#usage"},{"type":"extras","title":"Phoenix - Overview","doc":"For [Phoenix](https://www.phoenixframework.org/) it would look slightly different:\n\n```elixir\n  forward(\"/iiif/v3\", IIIFImagePlug.V3, %{\n    identifier_to_path_callback: &ImageStore.identifier_to_path/1\n  })\n```\n\nThe option `:identifier_to_path_callback` lets the plug map the IIIF [identifier](https://iiif.io/api/image/3.0/#21-image-request-uri-syntax) to an actual file path in your file system. \n\n`ImageStore.identifier_to_path/1` in this case might look something like this:\n\n```elixir\n  def identifier_to_path(identifier) do\n    \"/mnt/my_app_images/#{identifier}\"\n  end\n```\n\nA GET request `/iiif/v3/sample_image.jpg/info.json` would then cause the plug to look for an image file at `/mnt/my_app_images/sample_image.jpg` and return its metadata.\n\nFor the complete list of plug options have a look at the module documentation.","ref":"readme.html#phoenix"},{"type":"extras","title":"CORS - Overview","doc":"For your service to be fully implement the API specification, you need to properly configure Cross-Origin Resource Sharing (CORS). This has to be done outside the context of this plug, one option could be to use the\n[CorsPlug](https://hexdocs.pm/cors_plug/readme.html) library:\n\n```elixir\n(..)\n  plug(CORSPlug, origin: [\"*\"])\n  plug(:match)\n  plug(:dispatch)\n\n  forward(\"/\",\n    to: IIIFImagePlug.V3,\n    init_opts: %{\n      (..)\n    }\n  )\nend\n(..)\n```","ref":"readme.html#cors"},{"type":"extras","title":"Testing your endpoint - Overview","doc":"Because this plug is just a library and only part of your overall application, you might want to test your service's IIIF compliance against the official validator:\n- https://iiif.io/api/image/validator (web based)\n- https://github.com/IIIF/image-validator (repository with python based validator)","ref":"readme.html#testing-your-endpoint"},{"type":"extras","title":"Development - Overview","doc":"This repository comes with a minimalistic server, run the server with:\n\n```\niex -S mix run\n```\n\nThe metadata of the main sample file can now be accessed at http://127.0.0.1:4000/bentheim.jpg/info.json:\n\n```json\n{\n    \"id\": \"http://localhost:4000/bentheim.jpg\",\n    \"profile\": \"level2\",\n    \"type\": \"ImageServer3\",\n    \"protocol\": \"http://iiif.io/api/image\",\n    \"rights\": \"https://creativecommons.org/publicdomain/zero/1.0/\",\n    \"width\": 3000,\n    \"height\": 2279,\n    \"@context\": \"http://iiif.io/api/image/3/context.json\",\n    \"maxHeight\": 10000,\n    \"maxWidth\": 10000,\n    \"maxArea\": 100000000,\n    \"extra_features\": [\n        \"mirroring\",\n        \"regionByPct\",\n        \"regionByPx\",\n        \"regionSquare\",\n        \"rotationArbitrary\",\n        \"sizeByConfinedWh\",\n        \"sizeByH\",\n        \"sizeByPct\",\n        \"sizeByW\",\n        \"sizeByWh\",\n        \"sizeUpscaling\"\n    ],\n    \"preferredFormat\": [\n        \"jpg\"\n    ],\n    \"extraFormats\": [\n        \"webp\",\n        \"png\",\n        \"tif\"\n    ],\n    \"extraQualities\": [\n        \"color\",\n        \"gray\",\n        \"bitonal\"\n    ]\n}\n```\n\nThe image data of the sample file can be viewed at http://127.0.0.1:4000/bentheim.jpg/full/max/0/default.jpg and you are able to play around with the IIIF API parameters.","ref":"readme.html#development"},{"type":"extras","title":"Performance considerations","doc":"# Performance considerations","ref":"performance_considerations.html"},{"type":"extras","title":"Input - Performance considerations","doc":"","ref":"performance_considerations.html#input"},{"type":"extras","title":"Pyramid TIF - Performance considerations","doc":"The performance is greatly improved if you provide your images in more than one resolution. This can be accomplished by providing image pyramids. Using vips8 you can generate a TIF file pyramid by running.\n\n```bash\nvips tiffsave input.jpg output_pyramid.tif --compression deflate --tile --tile-width 256 --tile-height 256 --pyramid \n```\n\nThe same can be achieved in Elixir with [Vix](https://hex.pm/packages/vix):\n\n```elixir\n{:ok, file} = Image.new_from_file(\"input.jpg\")\n\nOperation.tiffsave(file, \"output_pyramid.tif\",\n    pyramid: true,\n    \"tile-height\": 256,\n    \"tile-width\": 256,\n    tile: true,\n    compression: :VIPS_FOREIGN_TIFF_COMPRESSION_DEFLATE\n)\n```\n\n\nThis will generate a single file that contains multiple pages of decreasing resolution of the base image:\n![Image pyramid example image](image_pyramid.png)\n\nThe IIIF Image plug will automatically evaluate these pages. If a downscaled version of the image is being requested, the plug will automatically pick the next highest resolution page for the requested scale (instead of always scaling down based on the maximum resolution).","ref":"performance_considerations.html#pyramid-tif"},{"type":"extras","title":"Output - Performance considerations","doc":"","ref":"performance_considerations.html#output"},{"type":"extras","title":"TIF - Performance considerations","doc":"If you want to allow the user to request tif files, be aware these can not be streamed and are currently buffered in memory before they are sent to the client. This might cause memory issues for very large files.","ref":"performance_considerations.html#tif"},{"type":"extras","title":"Alternatives to this plug - Performance considerations","doc":"This plug aims to implement the \"level 2\" [compliance](https://iiif.io/api/image/3.0/compliance) for the IIIF image API. \n\nIf you only want to provide \"level 0\" data (the most basic required for tiled viewers), you can preprocess your input images beforehand and serve them as static assets (without this library). There exist several [resources](https://training.iiif.io/dhsi/day-one/level-0-static.html) on how to do this.\n\nWorking with vips, you can run:\n\n```bash\nvips dzsave input.jpg preprocessed_out --layout iiif3 --depth onetile --overlap 0 --suffix .jpg\n```\n\nThis will create a directory \"preprocessed_out\" that contains a bunch of directories (each corresponding to a possible \"region\" parameter) and a `info.json` file. These can be served as static assets. You would need to update the \"id\" value to match your service:\n\n```json\n{\n  \"@context\": \"http://iiif.io/api/image/3/context.json\",\n  \"id\": \"https://example.com/iiif/preprocessed_out\",\n  \"type\": \"ImageService3\",\n  \"profile\": \"level0\",\n  \"protocol\": \"http://iiif.io/api/image\",\n  \"tiles\": [\n    {\n      \"scaleFactors\": [\n        1,\n        2,\n        4\n      ],\n      \"width\": 512\n    }\n  ],\n  \"width\": 3000,\n  \"height\": 2279\n}\n```\n\nAgain, Vix provides the same functionality in Elixir with [Vix.Vips.Operation.dzsave/3](https://hexdocs.pm/vix/Vix.Vips.Operation.html#dzsave/3).","ref":"performance_considerations.html#alternatives-to-this-plug"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.4.0 - Changelog","doc":"Made some minor additional changes to align with IIIF image API spec:\n- Fixed the static value for key \"type\" in the `info.json`.\n- Return 404 status code for invalid paths (instead of the 400 in the initial implementation).\n- Redirect to `info.json` if the request ends with the identifier (i.e. from `/path/to/plug/my-image` to `/path/to/plug/my-image/info.json`).\n- Reintroduced override options `:port`, `:host` and `:scheme` for proxy scenarios.","ref":"changelog.html#0-4-0"},{"type":"extras","title":"0.3.1 - Changelog","doc":"Fixed error with pyramid page optimization was not applied if no `vscale` was requested. This caused a fallback to full image processing.","ref":"changelog.html#0-3-1"},{"type":"extras","title":"0.3.0 - Changelog","doc":"Removed the `:scheme`, `:host`, `:port` and `:prefix` options, those are now evaluated directly from the [`%Plug.Conn{}`](https://hexdocs.pm/plug/Plug.Conn.html).","ref":"changelog.html#0-3-0"},{"type":"extras","title":"0.2.0 - Changelog","doc":"Added optimization for pyramid tiffs:\n- The `info.json` may now contain the [sizes](https://iiif.io/api/image/3.0/#53-sizes) property, if the identifier refers to a pyramid image.\n- When image data is requested based off an pyramid image, the plug now should select the best matching image from the pyramid and apply an adjusted transformation on that preprocessed lower resolution image (instead of always working on the maximum resolution).","ref":"changelog.html#0-2-0"},{"type":"extras","title":"0.1.0 - Changelog","doc":"Minimal viable v3 plug.","ref":"changelog.html#0-1-0"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"{}\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright {yyyy} {name of copyright owner}\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}